/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright 2023 SiFive, Inc.
 */

#ifndef __RISCV_ATOMIC_H
#define __RISCV_ATOMIC_H

/* use the generic asm/atomic.h until we define a better one */

#include <asm/system.h>
#include <asm-generic/atomic.h>

#endif
