/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (C) 2023 Marek Vasut <marex@denx.de>
 */
#ifndef ARM11_H
#define ARM11_H

#ifndef __ASSEMBLY__
void arm11_arch_cp15_allow_unaligned(void);
#endif /* ! __ASSEMBLY__ */

#endif	/* ARM11_H */
