/* SPDX-License-Identifier: GPL-2.0+ OR BSD-3-Clause */
/*
 * Copyright (C) 2024, STMicroelectronics - All Rights Reserved
 */

#ifndef __MACH_STM32MP_H_
#define __MACH_STM32MP_H_

int optee_get_reserved_memory(u32 *start, u32 *size);

#endif
