/* SPDX-License-Identifier: GPL-2.0-or-later */
/*
 * Copyright 2022 Toradex
 */

#ifndef __LPDDR4_TIMING_H__
#define __LPDDR4_TIMING_H__

void lpddr4_single_rank_training_patch(void);

#endif /* __LPDDR4_TIMING_H__ */
