/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (C) Marvell International Ltd. and its affiliates
 */

#ifndef _MV_DDR4_TRAINING_LEVELING_H
#define _MV_DDR4_TRAINING_LEVELING_H

int mv_ddr4_dynamic_wl_supp(u32 dev_num);

#endif /* _MV_DDR4_TRAINING_LEVELING_H */
