/* SPDX-License-Identifier: GPL-2.0+ */
/*
 * Copyright (c) 2024, Kongyang Liu <seashell11234455@gmail.com>
 */

#ifndef __CV1800B_ETHERNET_H
#define __CV1800B_ETHERNET_H

void cv1800b_ephy_init(void);

#endif
