/* SPDX-License-Identifier: GPL-2.0-or-later */

#ifndef __ASM_ACPI_TABLE_H__
#define __ASM_ACPI_TABLE_H__

/*
 * This file is needed by some drivers.
 * We will fill it when adding ACPI support for RISC-V.
 */

#endif /* __ASM_ACPI_TABLE_H__ */
